#!/bin/sh -x

# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: MIT-0
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this
# software and associated documentation files (the "Software"), to deal in the Software
# without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

# $1 must be instance role
declare -A AIRFLOW_ROLES=( [scheduler]=1 [worker]=1 [all]=1 )

if [ "$#" != "12" ]
then
        echo "Call this script with [all|scheduler|worker] DB_Root_User RDS_Root_User_Password_Label_SSM  DB_Airflow_User Airflow_Admin_User_Password_Label_SSM DB_Name DB_Host S3_Location_for_Airflow_Bootstrap_Scripts _SSM AWS_Region Redis_Endpoint EFS_Mount_Target Genie_Endpoint"
	exit 1
fi

if [ "${AIRFLOW_ROLES[$1]}" != "1" ] 
then 
	echo "Please select the correct Airflow function for this instance. Use all in doubt."
	exit 1
fi

export AIRFLOW_ROLE=${1}
export DB_ROOT_USER=${2}
export SSM_PARAMETER_STORE_LABEL_ROOT_USER_PASSWORD=${3}
export DB_AIRFLOW_USER=${4}
export SSM_PARAMETER_STORE_LABEL_AIRFLOW_ADMIN_PASSWORD=${5}
export DB_NAME=${6}
export DB_HOST=${7}
export BOOTSTRAP_BUCKET=${8}
export AWS_REGION=${9}
export REDIS_ENDPOINT=${10}
export EFS_MOUNT_TARGET=${11}
export GENIE_ENDPOINT=${12}

/usr/local/bin/pip3 install awscli install --upgrade --user awscli

export DB_ROOT_PASSWORD=(`aws ssm get-parameter --name  ${SSM_PARAMETER_STORE_LABEL_ROOT_USER_PASSWORD}  --region  ${AWS_REGION} --with-decryption --query Parameter.Value  --output text | tr '\t' ' '`)

export DB_AIRFLOW_ADMIN_PASSWORD=(`aws ssm get-parameter --name  ${SSM_PARAMETER_STORE_LABEL_AIRFLOW_ADMIN_PASSWORD}  --region  ${AWS_REGION} --with-decryption --query Parameter.Value  --output text | tr '\t' ' '`)

if [ -z "${DB_AIRFLOW_ADMIN_PASSWORD}" ] || [ -z "${DB_ROOT_PASSWORD}" ]
then
	echo "Check IAM Role Permissions, Access to SSM Parameter Store is required"
	exit 1
fi


SMTP_HOST="localhost"
SMTP_STARTTLS="True"
SMTP_SSL="False"
SMTP_USER="airflow"
SMTP_PORT="25"
SMTP_PASSWORD="airflow"
SMTP_MAIL_FROM="airflow@airflow.com"
GENIE_PORT="8080"

# Constants
export AIRFLOW_HOME="/etc/airflow"
export PATH=$PATH:/usr/local/bin

DAGS_FOLDER="${AIRFLOW_HOME}/dags"
LIBS_FOLDER="${AIRFLOW_HOME}/libs"
PLUGINS_FOLDER="${AIRFLOW_HOME}/plugins"
SCRIPTS_FOLDER="${AIRFLOW_HOME}/scripts"
AIRFLOW_USER="airflow"
AIRFLOW_LOG_DIR="${AIRFLOW_HOME}/logs"
TMP_DIR="/tmp/airflow"
EFS_MOUNTPOINT="/genieairflow/"
EFS_AIRFLOW_FOLDER="/genieairflow/airflow"
S3_LOG_LOCATION="s3://${BOOTSTRAP_BUCKET}/airflow/logs"
S3_LOG_CONNECTION_ID="s3_default"


# The function populates jinja templates from Python parameters dictionary
# $1 - jinja template file
# $2 - python dictionary structure with substitution parameters { "param" : "value", ... }
# $3 - output file
update_jinja_templates () {
	local TEMPLATE=$1
	local PARAMS="${2}"
	local FILE=$3
	cat << EOF | python > $FILE
import jinja2, json
params = json.loads('''$PARAMS''')
print jinja2.Template(open('$TEMPLATE').read()).render(params)
EOF
}


# Install packages
yum -y groupinstall "Development tools"
yum -y install gcc libffi-devel openssl-devel postgresql-devel mysql-devel python3 python3-devel 

#install pip
curl -o /tmp/get-pip.py https://bootstrap.pypa.io/get-pip.py
python3 /tmp/get-pip.py
/usr/local/bin/pip3 install pip --upgrade
hash -r
/usr/local/bin/pip3 install psycopg2 --upgrade
/usr/local/bin/pip3 install python-gnupg --upgrade
/usr/local/bin/pip3 install nflx-genie-client --upgrade
/usr/local/bin/pip3 install redis --upgrade
/usr/local/bin/pip3 install ldap3 --upgrade
/usr/local/bin/pip3 install flask_bcrypt --upgrade
/usr/local/bin/pip3 install jinja2 --upgrade
/usr/local/bin/pip3 install cryptography --upgrade
/usr/local/bin/pip3 install boto3 --upgrade

/usr/local/bin/pip3 install --ignore-installed lockfile
/usr/local/bin/pip3 install --ignore-installed chardet
/usr/local/bin/pip3 install --ignore-installed six

/usr/local/bin/pip3 install apache-airflow --upgrade
/usr/local/bin/pip3 install apache-airflow[async] --upgrade
/usr/local/bin/pip3 install apache-airflow[devel] --upgrade
/usr/local/bin/pip3 install apache-airflow[celery] --upgrade
/usr/local/bin/pip3 install apache-airflow[crypto] --upgrade
/usr/local/bin/pip3 install apache-airflow[jdbc] --upgrade
/usr/local/bin/pip3 install apache-airflow[hdfs] --upgrade
/usr/local/bin/pip3 install apache-airflow[ldap] --upgrade
/usr/local/bin/pip3 install apache-airflow[mysql] --upgrade
/usr/local/bin/pip3 install apache-airflow[password] --upgrade
/usr/local/bin/pip3 install apache-airflow[postgres] --upgrade
/usr/local/bin/pip3 install apache-airflow[s3] --upgrade

# airflow fails with eventlet 0.21.0
/usr/local/bin/pip3 install eventlet==0.24.1

# Add service user for Airflow
useradd -r -s /sbin/nologin -d ${AIRFLOW_HOME} ${AIRFLOW_USER}

# Stop iptables
service iptables stop
chkconfig iptables off

# Sync files from S3
aws s3 sync s3://${BOOTSTRAP_BUCKET}/airflow/bootstrap ${TMP_DIR}/bootstrap --delete --region ${AWS_REGION}

# Make scripts executable
chmod +x ${TMP_DIR}/bootstrap/*.sh -R

# Create local directories
install -o ${AIRFLOW_USER} -g ${AIRFLOW_USER} -d ${SCRIPTS_FOLDER}
install -o ${AIRFLOW_USER} -g ${AIRFLOW_USER} -d ${AIRFLOW_LOG_DIR}
install -o ${AIRFLOW_USER} -g ${AIRFLOW_USER} -d ${EFS_MOUNTPOINT}
ln -s ${AIRFLOW_LOG_DIR} -T /var/log/airflow

# Generate Fernet Key
export NEW_FKEY=`python3 -c 'from cryptography.fernet import Fernet; print(Fernet.generate_key().decode())'`


# Create airflow.cfg file
TEMPLATE=${TMP_DIR}/bootstrap/airflow.cfg.jinja
FILE=${AIRFLOW_HOME}/airflow.cfg
PARAMS=$(cat <<EOF
{
	"airflow_home" : "${AIRFLOW_HOME}",
	"sql_alchemy_conn" : "postgres://${DB_AIRFLOW_USER}:${DB_AIRFLOW_ADMIN_PASSWORD}@${DB_HOST}:5432/${DB_NAME}",
	"broker_url" : "redis://${REDIS_ENDPOINT}",
	"result_backend" : "db+postgresql://${DB_AIRFLOW_USER}:${DB_AIRFLOW_ADMIN_PASSWORD}@${DB_HOST}:5432/${DB_NAME}",
	"dags_folder" : "${DAGS_FOLDER}",
	"base_log_folder" : "${AIRFLOW_LOG_DIR}",
	"remote_base_log_folder" : "${S3_LOG_LOCATION}",
	"remote_log_conn_id" : "${S3_LOG_CONNECTION_ID}",
	"smtp_host" : "${SMTP_HOST}",
	"smtp_starttls" : "${SMTP_STARTTLS}",
	"smtp_ssl" : "${SMTP_SSL}",
	"smtp_user" : "${SMTP_USER}",
	"smtp_port" : "${SMTP_PORT}",
	"smtp_password" : "${SMTP_PASSWORD}",
	"smtp_mail_from" : "${SMTP_MAIL_FROM}",
	"fernet_key": "${NEW_FKEY}"
}
EOF
)
update_jinja_templates "$TEMPLATE" "$PARAMS" "$FILE"


# Sync DAGs to EFS to be shared across Airflow instances
install -o ${AIRFLOW_USER} -g ${AIRFLOW_USER} -d ${EFS_MOUNTPOINT}
mount -t nfs4 -o nfsvers=4.1,rsize=1048576,wsize=1048576,hard,timeo=600,retrans=2 ${EFS_MOUNT_TARGET}:/ ${EFS_MOUNTPOINT}

install -o ${AIRFLOW_USER} -g ${AIRFLOW_USER} -d ${EFS_AIRFLOW_FOLDER}/dags
install -o ${AIRFLOW_USER} -g ${AIRFLOW_USER} -d ${EFS_AIRFLOW_FOLDER}/libs
install -o ${AIRFLOW_USER} -g ${AIRFLOW_USER} -d ${EFS_AIRFLOW_FOLDER}/plugins

ln -s ${EFS_AIRFLOW_FOLDER}/dags -T ${DAGS_FOLDER}
ln -s ${EFS_AIRFLOW_FOLDER}/libs -T ${LIBS_FOLDER}
ln -s ${EFS_AIRFLOW_FOLDER}/plugins -T ${PLUGINS_FOLDER}



if [ "${AIRFLOW_ROLE}" == "scheduler" ] || [ "${AIRFLOW_ROLE}" == "all" ]
then

# Create airflow database and user
export PGHOST=${DB_HOST}
export PGUSER=${DB_ROOT_USER}
export PGPASSWORD=${DB_ROOT_PASSWORD}
createdb ${DB_NAME}
psql -d ${DB_NAME} -c "CREATE USER ${DB_AIRFLOW_USER} WITH PASSWORD '${DB_AIRFLOW_ADMIN_PASSWORD}'"
psql -d ${DB_NAME} -c "GRANT ALL PRIVILEGES ON DATABASE ${DB_NAME} to ${DB_AIRFLOW_USER}"

# this uses DB_* values from config file
/usr/local/bin/airflow initdb

# Set Variable s3_location_genie_demo to the Airflow BootStrap Bucket. This variable is used by the Demo Dags
/usr/local/bin/airflow variables --set s3_location_genie_demo ${BOOTSTRAP_BUCKET}


# add default Airflow connections: airlow_db, genie, s3_default
cat << EOF | python3
from airflow import settings
from airflow import models
session = settings.Session()
C = models.Connection
objects = session.query(C).filter(C.conn_id.in_(('airflow_db','s3_default','genie'))).all()
for object in objects:
  session.delete(object)
session.add( models.Connection( conn_id='airflow_db', conn_type='postgres', host='${DB_HOST}', login='${DB_AIRFLOW_USER}', password='${DB_AIRFLOW_ADMIN_PASSWORD}', schema='${DB_NAME}'))
session.add( models.Connection( conn_id='genie', conn_type='http', host='${GENIE_ENDPOINT}', port=${GENIE_PORT}))
session.add( models.Connection( conn_id='s3_default', conn_type='s3', extra='{ "config_format" : "boto" }'))
session.commit()
EOF


# Create cron file to sync DAGs	
cp ${TMP_DIR}/bootstrap/sync_dags.sh ${AIRFLOW_HOME}/scripts/sync_dags.sh
${AIRFLOW_HOME}/scripts/sync_dags.sh ${BOOTSTRAP_BUCKET} ${AWS_REGION} ${DAGS_FOLDER} ${LIBS_FOLDER} ${PLUGINS_FOLDER}

cat > ${AIRFLOW_HOME}/scripts/cronfile.txt <<EOF
*/5 * * * * sh ${AIRFLOW_HOME}/scripts/sync_dags.sh ${BOOTSTRAP_BUCKET} ${AWS_REGION} ${DAGS_FOLDER} ${LIBS_FOLDER} ${PLUGINS_FOLDER}
EOF
crontab -u ${AIRFLOW_USER} ${AIRFLOW_HOME}/scripts/cronfile.txt
fi

# Change file ownership
chown ${AIRFLOW_USER}:${AIRFLOW_USER} -R ${DAGS_FOLDER}
chown ${AIRFLOW_USER}:${AIRFLOW_USER} -R ${AIRFLOW_HOME}


BASE_PATH=$(pwd)
#get systemd files
cd ${TMP_DIR}/bootstrap
unzip systemd.zip
cd ${TMP_DIR}/bootstrap/systemd/
cp *.service /etc/systemd/system/
cp airflow /etc/sysconfig/airflow
chmod o+r /etc/sysconfig/airflow
systemctl daemon-reload

case "$AIRFLOW_ROLE" in
scheduler)
	systemctl start airflow-scheduler.service
	;;
worker)
	systemctl start airflow-worker.service
	systemctl start airflow-webserver.service
	systemctl start airflow-flower.service
	;;
all)
	systemctl start airflow-scheduler.service
	systemctl start airflow-worker.service
        systemctl start airflow-webserver.service
        systemctl start airflow-flower.service
	;;
*)
        echo "UNEXPECTED AIRFLOW_ROLE $AIRFLOW_ROLE"
        exit 1
        ;;
esac

# Cleanup
rm -rf ${TMP_DIR}

