# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: MIT-0
# Permission is hereby granted, free of charge, to any person obtaining a copy of this
# software and associated documentation files (the "Software"), to deal in the Software
# without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from airflow import DAG
from airflow.operators import GenieOperator
from datetime import datetime, timedelta
from mako.pyparser import arg_id
from airflow.models import Variable

# DAG specific parameters and variables
dag_name = 'movielens_transform_to_parquet'
start_date = datetime(2019, 10, 27)  # set to latest possible to avoid many backfills
schedule_interval = timedelta(hours=1) #every hour
genie_conn_id = 'genie'  # Automatically Added During Airflow's Instasllation via Cloudformation Template. See Airflow > Admin > Connections.

bucket= Variable.get('s3_location_genie_demo')

# Airflow default DAG parameters ( class DAG: https://github.com/apache/incubator-airflow/blob/10d70d9d7e9e49b541a0817dd48f7fd35a634675/airflow/models.py )
dag_default_args = {
    'owner': 'hadoop',
    'airflow_conn_id' : genie_conn_id,
    'depends_on_past': False,
    'wait_for_downstream': False,
    'start_date': start_date,
    'email': 'empty@customcorp.custom',
    'email_on_failure': True,
    'email_on_retry': False,
    'retries': 0,
    'retry_delay': timedelta(seconds=60),
    'catchup': False
    }


# Initialize the DAG
# Concurrency --> Number of tasks allowed to run concurrently
extraction_dag = DAG(dag_name,
          default_args=dag_default_args,
          start_date=start_date,
          schedule_interval=schedule_interval,
          concurrency=3,
          max_active_runs=1
          )

spark_transform_to_parquet_movies = GenieOperator(
    task_id='transform_to_parquet_movies',
    cluster_tags=['emr.cluster.role:batch'],
    command_tags=['type:spark-submit', 'version:2.4.3'],
    command_arguments="transform_to_parquet.py s3://{}/airflow/demo/input/csv/{}  s3://{}/demo/output/parquet/{}/".format(bucket,'movies.csv',bucket,'movies'),
    dependencies="s3://{}/airflow/dag_artifacts/transforms/transform_to_parquet.py".format(bucket),
    description='Demo Spark Submit Job',
    job_name="Genie Demo Spark Submit Job",
    tags=['transform_to_parquet_movies'],
    xcom_vars=dict(),
    retries=3,
    dag=extraction_dag)

spark_transform_to_parquet_ratings = GenieOperator(
    task_id='transform_to_parquet_ratings',
    cluster_tags=['emr.cluster.role:batch'],
    command_tags=['type:spark-submit', 'version:2.4.3'],
    command_arguments="transform_to_parquet.py s3://{}/airflow/demo/input/csv/{}  s3://{}/demo/output/parquet/{}/".format(bucket,'ratings.csv',bucket,'ratings'),
    dependencies="s3://{}/airflow/dag_artifacts/transforms/transform_to_parquet.py".format(bucket),
    description='Demo Spark Submit Job',
    job_name="Genie Demo Spark Submit Job",
    tags=['transform_to_parquet_ratings'],
    xcom_vars=dict(),
    retries=3,
    dag=extraction_dag)


spark_transform_to_parquet_links = GenieOperator(
    task_id='transform_to_parquet_links',
    cluster_tags=['emr.cluster.role:batch'],
    command_tags=['type:spark-submit', 'version:2.4.3'],
    command_arguments="transform_to_parquet.py s3://{}/airflow/demo/input/csv/{}  s3://{}/demo/output/parquet/{}/".format(bucket,'links.csv',bucket,'links'),
    dependencies="s3://{}/airflow/dag_artifacts/transforms/transform_to_parquet.py".format(bucket),
    description='Demo Spark Submit Job',
    job_name="Genie Demo Spark Submit Job",
    tags=['transform_to_parquet_links'],
    xcom_vars=dict(),
    retries=3,
    dag=extraction_dag)



spark_transform_to_parquet_tags = GenieOperator(
    task_id='transform_to_parquet_tags',
    cluster_tags=['emr.cluster.role:batch'],
    command_tags=['type:spark-submit', 'version:2.4.3'],
    command_arguments="transform_to_parquet.py s3://{}/airflow/demo/input/csv/{}  s3://{}/demo/output/parquet/{}/".format(bucket,'tags.csv',bucket,'tags'),
    dependencies="s3://{}/airflow/dag_artifacts/transforms/transform_to_parquet.py".format(bucket),
    description='Demo Spark Submit Job',
    job_name="Genie Demo Spark Submit Job",
    tags=['transform_to_parquet_tags'],
    xcom_vars=dict(),
    retries=3,
    dag=extraction_dag)

spark_transform_to_parquet_genome_scores = GenieOperator(
    task_id='transform_to_parquet_genome_scores',
    cluster_tags=['emr.cluster.role:batch'],
    command_tags=['type:spark-submit', 'version:2.4.3'],
    command_arguments="transform_to_parquet.py s3://{}/airflow/demo/input/csv/{}  s3://{}/demo/output/parquet/{}/".format(bucket,'genome-scores.csv',bucket,'genome-scores'),
    dependencies="s3://{}/airflow/dag_artifacts/transforms/transform_to_parquet.py".format(bucket),
    description='Demo Spark Submit Job',
    job_name="Genie Demo Spark Submit Job",
    tags=['transform_to_parquet_genome_scores'],
    xcom_vars=dict(),
    retries=3,
    dag=extraction_dag)

spark_transform_to_parquet_genome_tags = GenieOperator(
    task_id='transform_to_parquet_genome_tags',
    cluster_tags=['emr.cluster.role:batch'],
    command_tags=['type:spark-submit', 'version:2.4.3'],
    command_arguments="transform_to_parquet.py s3://{}/airflow/demo/input/csv/{}  s3://{}/demo/output/parquet/{}/".format(bucket,'genome-tags.csv',bucket,'genome-tags'),
    dependencies="s3://{}/airflow/dag_artifacts/transforms/transform_to_parquet.py".format(bucket),
    description='Demo Spark Submit Job',
    job_name="Genie Demo Spark Submit Job",
    tags=['transform_to_parquet_genome_tags'],
    xcom_vars=dict(),
    retries=3,
    dag=extraction_dag)




spark_transform_to_parquet_ratings.set_upstream(spark_transform_to_parquet_movies)
spark_transform_to_parquet_links.set_upstream(spark_transform_to_parquet_movies)
spark_transform_to_parquet_tags.set_upstream(spark_transform_to_parquet_movies)
spark_transform_to_parquet_genome_scores.set_upstream(spark_transform_to_parquet_movies)
spark_transform_to_parquet_genome_tags.set_upstream(spark_transform_to_parquet_movies)
