#!/usr/bin/python3.7
# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: MIT-0
# Permission is hereby granted, free of charge, to any person obtaining a copy of this
# software and associated documentation files (the "Software"), to deal in the Software
# without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
import logging

from airflow.models import BaseOperator
from airflow.utils import apply_defaults
from airflow.plugins_manager import AirflowPlugin
from airflow.hooks.base_hook import BaseHook
from pygenie.jobs import GenieJob
from pygenie.conf import GenieConf

class GenieHook(BaseHook):

    def __init__(self, airflow_conn_id='genie'):
        self.airflow_conn_id = airflow_conn_id
        self.genie_endpoint = None
        self.genie_conf = None
        

    def get_conn(self):
        logging.info('Genie connection id:{airflow_conn_id}'.format(airflow_conn_id=self.airflow_conn_id))
        airflow_conn = self.get_connection(self.airflow_conn_id)

        if airflow_conn.host.startswith("http"):
            self.genie_endpoint = '{host}:{port}'.format(conn_type=airflow_conn.conn_type, host=airflow_conn.host, port=airflow_conn.port)
        else:
            self.genie_endpoint = '{schema}://{host}:{port}'.format(conn_type=airflow_conn.conn_type, host=airflow_conn.host, port=airflow_conn.port)
        logging.info('Genie connection info: genie_endpoint={}'.format(self.genie_endpoint))
        
        self.genie_conf = GenieConf()
        self.genie_conf.genie.url = self.genie_endpoint
        self.genie_conf.genie.username = 'hadoop'
        self.genie_conf.genie.version = '3.0.0'
        return self.genie_conf


class GenieOperator(BaseOperator):
    '''
    https://netflix.github.io/genie/docs/3.3.20/rest/#_api_docs
    class GenieJob(object):
        self._application_ids = list()
        self._archive = True
        self._cluster_tag_mapping = defaultdict(list)
        self._command_arguments = None
        self._command_options = defaultdict(dict)
        self._command_tags = list()
        self._dependencies = list()
        self._description = None
        self._email = None
        self._genie_cpu = None
        self._genie_memory = None
        self._group = None
        self._job_id = uuid_str()
        self._job_name = None
        self._job_version = 'NA'
        self._parameters = dict()
        self._post_cmd_args = list()
        self._setup_file = None
        self._tags = list()
        self._timeout = None
        self._username = self._conf.get('genie.username')  
        
    :param airflow_conn_id: Name of the Airflow connection id, default: genie.
    :type airflow_conn_id: string    

    :param cluster_tags: Tags used by Genie when selecting the clusters that will run the job. 
    :type cluster_tags: list

    :param command_arguments: Any arguments to append to the command executable when the job is run. e.g for hadoop: dfs -cp s3://bucket/src s3://bucket/dst
    :type command_arguments: string

    :param command_tags: Tags used by Genie when selecting which command to use when executing the job.
    :type command_tags: list

    :param dependencies: URIs of dependency files which will be downloaded into job working directory at runtime, e.g. s3://bucket/foo1,s3://bucket/foo2
    :type dependencies: list
        
    :param description:  Description of the job.
    :type description: string

    :param email: Email address that Genie uses to send a notification to after the job is done
    :type email: string 
         
    :param genie_memory: Set the amount of memory (MB) for Genie to allocate when executing the job.
    :type genie_memory: string 

    :param job_name: Name of job, default: the airflow task instance task_id
    :type job_name: string
    
    :param post_cmd_args: 
    :type post_cmd_args: list
          
    :param setup_file:  A location for any setup that needs to be done when installing
    :type setup_file: string

    :param tags: Adds a tag for the job. 
    :type tags: list

    :param timeout: The timeout (in seconds) after which job will be killed by system, system setting used if not set.   
    :type timeout: int

    :param xcom_vars: variables available to other tasks via xcom
    :type xcom_vars: dictionary
    '''
 
    
    template_fields = ('airflow_conn_id',
                       'cluster_tags',
                       'command_arguments',
                       'command_tags',
                       'dependencies',
                       'description',
                       'email',
                       'genie_memory',
                       'job_name',
                       'post_cmd_args',
                       'setup_file',
                       'tags',
                       'xcom_vars')
    ui_color = '#D8BFD8'

    @apply_defaults
    def __init__(self,
                airflow_conn_id=None,
                cluster_tags=list(),
                command_arguments=None,
                command_tags=list(),
                dependencies=list(),
                description=None,
                email=None,
                genie_memory=None,
                job_name=None,
                post_cmd_args=list(),
                setup_file=None,
                tags=list(),
                timeout=None,
                xcom_vars=dict(),
                *args, **kwargs):
        super(GenieOperator, self).__init__(*args, **kwargs)

        if not airflow_conn_id:
            raise ValueError('Must provide reference to Genie connection in Airflow!')      
        if type(cluster_tags) is not list or not cluster_tags:
            raise ValueError('Must provide cluster tags list!')         
        if type(command_tags) is not list or not command_tags:
            raise ValueError('Must provide command tags list!')         
        if not job_name:
            raise ValueError('Must provide job name!')    

        self.airflow_conn_id = airflow_conn_id
        self.cluster_tags = cluster_tags
        self.command_arguments = command_arguments
        self.command_tags = command_tags
        self.dependencies = dependencies
        self.description = description
        self.email = email
        self.genie_memory = genie_memory
        self.job_name = job_name
        self.post_cmd_args = post_cmd_args
        self.setup_file = setup_file
        self.tags = tags
        self.timeout = timeout
        self.xcom_vars = xcom_vars

        logging.debug('''Loaded Genie Operator with job info:
        airflow connection id: {}
        job name: {}
        description: {}
        cluster tags: {}
        command tags: {}
        command arguments: {}
        dependencies: {}
        setup file: {}
        tags: {}
        timeout: {}'''.format(self.airflow_conn_id,
                          self.job_name,
                          self.description,
                          ','.join(self.cluster_tags),
                          ','.join(self.command_tags),
                          self.command_arguments,
                          ','.join(self.dependencies),
                          self.setup_file,
                          ','.join(self.tags),
                          self.timeout))

    def execute(self, context):
        
        genie_conf = GenieHook().get_conn()
        job_definition = GenieJob(conf=genie_conf)          

        execution_date = context['execution_date']
        if self.cluster_tags:
            job_definition.cluster_tags(self.cluster_tags)
        if self.command_arguments:
            job_definition.command_arguments(self.command_arguments)
        if self.command_tags:
            job_definition.command_tags(self.command_tags)
        if self.dependencies:
            job_definition.dependencies(self.dependencies)
        if self.description:
            job_definition.description(self.description)
        if self.email:
            job_definition.genie_email(self.email)
        if self.genie_memory:
            job_definition.genie_memory(self.genie_memory)
        if self.job_name:
            job_definition.job_name(self.job_name + '_' + str(execution_date))
        if self.post_cmd_args:
            job_definition.post_cmd_args(self.post_cmd_args)
        if self.setup_file:
            job_definition.setup_file(self.setup_file)
        if self.tags:
            job_definition.tags(self.tags)
        if self.timeout:
            job_definition.genie_timeout(self.timeout)

        logging.info('Submitting job to Genie: ' + self.job_name)
        job = job_definition.execute()
        logging.info('Job with id {} has status {}'.format(job.job_id, job.status))
        logging.info(job.job_link)
        
        job.wait()
        logging.info('''Job with Id {} finished with status {}'''.format(job.job_id, job.status))

        if job.status != 'SUCCEEDED':
            raise Exception('''Genie job Failed!
            job status: {}
            status message: {}
            request data: {}'''.format(job.status, job.status_msg, str(job.request_data)))
        else:
            logging.info('''Genie job Succeeded!
            job status: {}
            status message: {}
            request data: {}
            stderr: {}
            stdout: {}
            genie log: {}'''.format(job.status,
                                  job.status_msg,
                                  str(job.request_data),
                                  job.stderr(),
                                  job.stdout(),
                                  job.genie_log()))

        logging.info('Pushing XComs: ' + ','.join([ '{k}={v}'.format(k=k, v=v) for k, v in self.xcom_vars.items() ]))
        for k, v in self.xcom_vars.items():
            self.xcom_push(context=context, key=k, value=v)

# see Airflow documentation for the structure of a custom Operator
class AirflowGeniePlugin(AirflowPlugin):
    name = 'genie_plugin'
    operators = [GenieOperator]
    hooks = [GenieHook]
