#!/bin/bash -x
# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: MIT-0
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this
# software and associated documentation files (the "Software"), to deal in the Software
# without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
###
# The script installs Genie Open Source Software by Netflix 
# Genie home directory: /opt/genie
# Genie service user: hadoop
# Genie service log file: /var/log/genie.log
# use Upstart to start stop service; e.g
#  sudo stop genie
#  sudo start genie

cd /tmp

if [ "$#" != "10" ]
then
	echo "Call this script with S3_Bucket_Name_With_Genie_Scripts IAM_Role_ARN_For_Genie_Application RDS_Root_User_Password_Label_SSM Genie_Admin_User_Password_Label_SSM DB_Root_User DB_Genie_Admin_User AWS_Region Zookeeper_Connection_String DB_Name DB_HOST"
        exit 1
fi

export S3_BUCKET=${1}
export GENIE_IAM_ROLE_ARN=${2}
export SSM_PARAMETER_STORE_LABEL_ROOT_USER_PASSWORD=${3}
export SSM_PARAMETER_STORE_LABEL_GENIE_ADMIN_PASSWORD=${4}
export DB_ROOT_USER=${5}
export DB_GENIE_USER=${6}
export AWS_REGION=${7}
export ZOOKEEPER_CONNECT_STRING=${8}
export DB_NAME=${9}
export DB_HOST=${10}

/usr/bin/local/pip install --upgrade pip
/usr/bin/local/pip install awscli install --upgrade --user awscli

export DB_ROOT_PASSWORD=(`aws ssm get-parameter --name  ${SSM_PARAMETER_STORE_LABEL_ROOT_USER_PASSWORD}  --region  ${AWS_REGION} --with-decryption --query Parameter.Value  --output text | tr '\t' ' '`)

export DB_GENIE_ADMIN_PASSWORD=(`aws ssm get-parameter --name  ${SSM_PARAMETER_STORE_LABEL_GENIE_ADMIN_PASSWORD}  --region  ${AWS_REGION} --with-decryption --query Parameter.Value  --output text | tr '\t' ' '`)


if [ -z "${DB_ROOT_PASSWORD}" ] || [ -z "${DB_GENIE_ADMIN_PASSWORD}" ]
then
        echo "Check IAM Role Permissions, Access to SSM Parameter Store is required"
        exit 1
fi

# Constants
# set user as hadoop - default for EMR
export GENIE_USER="hadoop"
export GENIE_GROUP=${GENIE_USER}
export GENIE_HOME="/opt/genie"
export GENIE_LOG_DIR="${GENIE_HOME}/log"
export GENIE_LOG="${GENIE_LOG_DIR}/genie.log"
export GENIE_ENDPOINT="http://localhost:8080"
export TMP_DIR="/tmp/genie"

rm -rf /opt/genie

### Install Java 8
yum -y install java-1.8.0-openjdk.x86_64
yum -y install java-1.8.0-openjdk-devel
alternatives --set java /usr/lib/jvm/jre-1.8.0-openjdk.x86_64/bin/java

### Install RPM packages
yum -y install postgresql95-devel postgresql-jdbc mysql wget nfs-utils xmlstarlet

### Install awscli and other python utilities
/usr/local/bin/pip install boto3 --upgrade
/usr/local/bin/pip install python-gnupg --upgrade
/usr/local/bin/pip install nflx_genie_client==3.6.10 --upgrade
/usr/local/bin/pip install pyyaml --upgrade

### Download the binary artifacts in a temp directory
mkdir -p ${TMP_DIR}
aws s3 sync s3://${S3_BUCKET}/genie/artifacts/ ${TMP_DIR}

### Create Genie app directories
mkdir -p ${GENIE_HOME}
mkdir -p ${GENIE_HOME}/lib
mkdir -p ${GENIE_HOME}/bin
mkdir -p ${GENIE_HOME}/config
mkdir -p ${GENIE_HOME}/tmp
mkdir -p ${GENIE_HOME}/tmp/archives
mkdir -p ${GENIE_HOME}/tmp/attachments
mkdir -p ${GENIE_HOME}/tmp/cache
mkdir -p ${GENIE_HOME}/tmp/jobs
mkdir -p ${GENIE_LOG_DIR}
ln -s ${GENIE_LOG_DIR} -T /var/log/genie


### Get necessary binaries for Genie and PostgresSQL. See Genie Docs for Genie Download URL https://netflix.github.io/genie/docs/3.3.20/reference/#_installation
cd ${TMP_DIR}
wget -O genie-app-3.3.20.jar  https://bintray.com/netflixoss/maven/download_file?file_path=com%2Fnetflix%2Fgenie%2Fgenie-app%2F3.3.20%2Fgenie-app-3.3.20.jar -P  ${TMP_DIR}
wget -O postgresql-42.2.6.jar https://jdbc.postgresql.org/download/postgresql-42.2.6.jar -P  ${TMP_DIR}

#Get Genie PostgresSQL DDL
wget -O V3_2_0__Base_Version.sql https://raw.githubusercontent.com/Netflix/genie/v3.3.20/genie-web/src/main/resources/db/migration/postgresql/V3_2_0__Base_Version.sql -P  ${TMP_DIR}

wget -O V3_3_0__Merge_Jobs_Tables.sql https://raw.githubusercontent.com/Netflix/genie/v3.3.20/genie-web/src/main/resources/db/migration/postgresql/V3_3_0__Merge_Jobs_Tables.sql  -P  ${TMP_DIR}



### Copy necessary jars to Genie lib directory
cp ${TMP_DIR}/genie-app-3.3.20.jar ${GENIE_HOME}/lib/
cp ${TMP_DIR}/postgresql-42.2.6.jar ${GENIE_HOME}/lib/

### Copy configuration files
cp ${TMP_DIR}/application.yml ${GENIE_HOME}/config/
cp ${TMP_DIR}/application-s3.yml ${GENIE_HOME}/config/
cp ${TMP_DIR}/application-jdbc.yml ${GENIE_HOME}/config/
cp ${TMP_DIR}/application-logging.yml ${GENIE_HOME}/config/

# Create Genie database and the Genie Admin User
export PGHOST=${DB_HOST}
export PGUSER=${DB_ROOT_USER}
export PGPASSWORD=${DB_ROOT_PASSWORD}
createdb ${DB_NAME}
psql -d ${DB_NAME} -c "CREATE USER ${DB_GENIE_USER} WITH PASSWORD '${DB_GENIE_ADMIN_PASSWORD}'"
psql -d ${DB_NAME} -c "GRANT ALL PRIVILEGES ON DATABASE ${DB_NAME} to ${DB_GENIE_USER}"
export PGUSER=${DB_GENIE_USER}
export PGPASSWORD=${DB_GENIE_ADMIN_PASSWORD}
#Load schema
cat ${TMP_DIR}/V3_2_0__Base_Version.sql |  grep -v -E '(CREATE\ EXTENSION|COMMENT\ ON)' > ${TMP_DIR}/V3_2_0__Base_Version_no_extensions.sql
psql ${DB_NAME} < ${TMP_DIR}/V3_2_0__Base_Version_no_extensions.sql
psql ${DB_NAME} < ${TMP_DIR}/V3_3_0__Merge_Jobs_Tables.sql

# Update config YAML files
cat << EOF | python
import yaml
with open("${GENIE_HOME}/config/application.yml","r+") as f:
    application = yaml.load(f)
    application["genie"]["file"]["cache"]["location"] = "file://${GENIE_HOME}/tmp/cache"
    application["genie"]["jobs"]["locations"]["archives"] = "file://${GENIE_HOME}/tmp/archives/"
    application["genie"]["jobs"]["locations"]["attachments"] = "file://${GENIE_HOME}/tmp/attachments/"
    application["genie"]["jobs"]["locations"]["jobs"] = "file://${GENIE_HOME}/tmp/jobs/"
    application["spring"]["cloud"]["cluster"]["zookeeper"]["connect"] = "${ZOOKEEPER_CONNECT_STRING}"
    application["spring"]["cloud"]["cluster"]["leader"]["enabled"] = "true"
    application["loader"] = {"path": "${GENIE_HOME}/lib"}
    f.seek(0)
    yaml.dump(application, f, default_flow_style=False)
    f.truncate()
with open("${GENIE_HOME}/config/application-jdbc.yml","r+") as f:
    application = yaml.load(f)
    application["spring"]["datasource"]["url"] = "jdbc:postgresql://${PGHOST}/${DB_NAME}"
    application["spring"]["datasource"]["username"] = "${DB_GENIE_USER}"
    application["spring"]["datasource"]["password"] = "${DB_GENIE_ADMIN_PASSWORD}"
    f.seek(0)
    yaml.dump(application, f, default_flow_style=False)
    f.truncate()
with open("${GENIE_HOME}/config/application-s3.yml","r+") as f:
    application = yaml.load(f)
    application["genie"]["aws"]["credentials"]["role"] = "${GENIE_IAM_ROLE_ARN}"
    f.seek(0)
    yaml.dump(application, f, default_flow_style=False)
    f.truncate()
EOF

### Stop iptables
service iptables stop
chkconfig iptables off

### Create start scripts
cat > ${GENIE_HOME}/bin/start-genie.sh <<EOF
# Needed for Genie applications setup scripts
export S3_BUCKET=${S3_BUCKET}
export AWS_REGION=${AWS_REGION} 
export JAVA_OPTS="-Djava.io.tmpdir=${GENIE_HOME}/tmp -Djava.net.preferIPv4Stack=true -Djava.net.preferIPv4Addresses=true"
# Default config directory is ${GENIE_HOME}/.genie but we use ${GENIE_HOME}/config/
exec java \${JAVA_OPTS} -jar ${GENIE_HOME}/lib/genie-app-3.3.20.jar --spring.config.location=${GENIE_HOME}/config/
EOF
chmod a+x ${GENIE_HOME}/bin/start-genie.sh

### Create upstart init script
cat > /etc/init/genie.conf <<EOF
description "Genie Server"

start on runlevel [2345]
stop on runlevel [!2345]
respawn
respawn limit 10 5

script
exec su -s /bin/sh -c 'exec "${GENIE_HOME}/bin/start-genie.sh"' ${GENIE_USER} > ${GENIE_LOG}
end script

post-stop script 
rm -rf ${GENIE_HOME}/tmp/*
end script
EOF

# Create Genie user
useradd -r -s /sbin/nologin -d ${GENIE_HOME} ${GENIE_USER}
chown -R ${GENIE_USER}:${GENIE_GROUP} ${GENIE_HOME}
chmod 777 /tmp

# Allow Genie user to run yum 
echo "${GENIE_USER} ALL=(ALL) NOPASSWD:/usr/bin/yum" > /etc/sudoers.d/genie

# Create /mnt/s3, /mnt1/s3 needed by EMR Hadoop
install -o ${GENIE_USER} -g ${GENIE_USER} -d /mnt/s3
install -o ${GENIE_USER} -g ${GENIE_USER} -d /mnt1/s3

# Cleanup
rm -rf ${TMP_DIR}

# start the service and wait for it to boot
start genie
tail -f ${GENIE_LOG} | grep -m1 "Started GenieWeb"

