#!/bin/bash -x
# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: MIT-0
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this
# software and associated documentation files (the "Software"), to deal in the Software
# without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

##### 
# Run this as an EMR Step to register cluster with Genie.
# The script will also upload all EMR libraries on S3
# Each application must have a YAML config in s3://${S3_BUCKET}/genie/applications/${APP_NAME}-${APP_VERSION}.yml
# Application name must match EMR naming under /etc/ directory
# Application version must be the same or compatible with the EMR version
#
#Steps of this Script:
# Gets Cluster information from metadata and system files.
# Copies cluster configurations to Amazon S3 in order for Genie to use.
# Registers Cluster with Genie.

if [ "$#" -ne 2 ]
then
	echo -e "Usage: $0 s3_bucket_name [genie_endpoint]\n"
	echo -e "  Example:"
	echo -e "  $0 mybucket http://genie.example.com:8080" 
	exit 1
fi

IS_MASTER=$(jq -r '.isMaster' /mnt/var/lib/info/instance.json)
if [ "${IS_MASTER}" != "true" ]
then
	echo "Hostname $(hostname) is not a master ... exiting"
	exit 0
fi

# Grab command line parameters
# e.g. mybucket
S3_BUCKET=$1 
# http://genie.example.com:8080
GENIE_ENDPOINT=$2

REGION=$(curl -s http://169.254.169.254/latest/meta-data/placement/availability-zone/|sed 's/.$//')
EMR_RELEASE=$(perl -ne '/releaseLabel: "(.*)"/ && print $1;' /mnt/var/lib/info/job-flow-state.txt)
CLUSTER_ID=$(jq -r '.jobFlowId' /mnt/var/lib/info/job-flow.json)
CLUSTER_NAME=$(aws emr describe-cluster --cluster-id ${CLUSTER_ID} --region ${REGION}  --query 'Cluster.Name' --output text)
CLUSTER_ROLE="batch"
HADOOP_VERSION=$(hadoop version | head -1 | perl -ne '/^Hadoop (\d.\d.\d).*/ && print $1')
S3_CLUSTER_LOCATION=s3://${S3_BUCKET}/genie/clusters/${CLUSTER_ID}
HADOOP_APPS=$(aws emr describe-cluster --cluster-id ${CLUSTER_ID} --region ${REGION} --query "Cluster.Applications" --output text | sed 's/\s/-/g' | tr '[:upper:]' '[:lower:]')

# parse applications string and populate array with applications
declare -a APPLICATIONS
HADOOP_APPLICATIONS=(${HADOOP_APPS//,/ })

echo "Environment info:"
echo "	MASTER $(hostname)"
echo "	CLUSTER_ID $CLUSTER_ID"
echo "	CLUSTER_NAME $CLUSTER_NAME"
echo "	EMR_RELEASE" $EMR_RELEASE
echo "	S3_CLUSTER_LOCATION" $S3_CLUSTER_LOCATION
echo "  HADOOP_APPLICATIONS" $HADOOP_APPS

# Download EMR libs; The Genie hadoop application setup.sh script will download and put them in the spark/jars directory
aws s3 sync --region ${REGION} /usr/share/aws ${S3_CLUSTER_LOCATION}/aws/ --exclude "*" --include "*aws-java-sdk*" --include "*emr/goodies*" --include "*emr/ddb*" --include "*emr/emrfs*" --include "*emr/kinesis*" --include "*emr/redshift*" --include "*emr/s3-dist-cp*"
aws s3 cp --region ${REGION} /usr/lib/hadoop/hadoop-aws.jar ${S3_CLUSTER_LOCATION}/jars/hadoop-aws.jar
yumdownloader hadoop-lzo-0.4.19-1.amzn1.x86_64 --destdir=/tmp/rpm/
aws s3 cp --region ${REGION} /tmp/rpm/hadoop-lzo-0.4.19-1.amzn1.x86_64.rpm ${S3_CLUSTER_LOCATION}/rpm/

echo "Upload EMR cluster applications configurations to S3"
for HADOOP_APPLICATION in "${HADOOP_APPLICATIONS[@]}"
do
	APP_NAME=${HADOOP_APPLICATION//-*/}
	APP_VERSION=${HADOOP_APPLICATION//*-/}
	aws s3 cp --region ${REGION} --exclude=* --include "*.xml" --include "*.properties" --include "*.conf" --recursive /etc/${APP_NAME}/conf/ ${S3_CLUSTER_LOCATION}/conf/${APP_NAME}
done

echo "Downloading genie_register_cluster.py"
aws s3 cp --region ${REGION} s3://${S3_BUCKET}/genie/scripts/genie_register_cluster.py ./
sudo pip install boto3 --upgrade
sudo /usr/bin/pip install nflx_genie_client==3.6.10 --upgrade

echo "Generating Genie cluster configuration file"
YAML_FILE="${CLUSTER_ID}.yml"
cat > ./${YAML_FILE} <<EOF
id: ${CLUSTER_ID}
name: ${CLUSTER_NAME}
user: hadoop
version: ${HADOOP_VERSION}
status: UP
tags:
  - emr.cluster.release:${EMR_RELEASE}
  - emr.cluster.id:${CLUSTER_ID}
  - emr.cluster.name:${CLUSTER_NAME}
  - emr.cluster.role:${CLUSTER_ROLE}
configs:
  - ${S3_CLUSTER_LOCATION}/conf/hadoop/core-site.xml
  - ${S3_CLUSTER_LOCATION}/conf/hadoop/mapred-site.xml
  - ${S3_CLUSTER_LOCATION}/conf/hadoop/yarn-site.xml
EOF

aws s3 cp --region ${REGION} ${YAML_FILE} ${S3_CLUSTER_LOCATION}/${YAML_FILE}

# Register Cluster with Genie during cluster startup
 python27 ./genie_register_cluster.py \
 ${YAML_FILE} \
 ${S3_BUCKET} \
 ${REGION} \
 ${GENIE_ENDPOINT}

echo "Creating unregister bootstrap action in /mnt/var/lib/instance-controller/public/shutdown-actions/"
sudo mkdir -p /mnt/var/lib/instance-controller/public/shutdown-actions/
# see http://docs.aws.amazon.com/ElasticMapReduce/latest/DeveloperGuide/emr-plan-bootstrap.html#bootstrap_Shutown
sudo /bin/bash -c "cat > /mnt/var/lib/instance-controller/public/shutdown-actions/genie_unregister.sh" <<EOF
#!/bin/bash
aws s3 cp --region ${REGION} s3://${S3_BUCKET}/genie/scripts/genie_unregister_cluster.py genie_unregister_cluster.py
python genie_unregister_cluster.py "${CLUSTER_ID}" "${GENIE_ENDPOINT}"
EOF
sudo chmod a+x /mnt/var/lib/instance-controller/public/shutdown-actions/genie_unregister.sh

exit 0
